Package: DiceKriging
Title: Kriging Methods for Computer Experiments
Version: 1.6.0
Date: 2021-02-23
Author: Olivier Roustant, David Ginsbourger, Yves Deville. Contributors: Clement Chevalier, Yann Richet.
Maintainer: Olivier Roustant <roustant@insa-toulouse.fr>
Description: Estimation, validation and prediction of kriging models. Important functions : km, print.km, plot.km, predict.km.
Depends: methods
Suggests: rgenoud (>= 5.8-2.0), foreach, doParallel, testthat, numDeriv
License: GPL-2 | GPL-3
URL: https://dicekrigingclub.github.io/www/
NeedsCompilation: yes
Packaged: 2021-02-23 17:02:18 UTC; yves
Repository: CRAN
Date/Publication: 2021-02-23 17:30:03 UTC
Built: R 4.4.0; x86_64-apple-darwin17.7.0; 2024-04-28 23:13:46 UTC; unix
